// Type definitions for react-paginate 2.1.3
// Project: https://github.com/AdeleD/react-paginate
// Definitions by: Simon Hartcher <https://github.com/deevus>
// Definitions: https://github.com/DefinitelyTyped/DefinitelyTyped

declare module "react-paginate" {
    import * as React from "react";

    interface ReactPaginateProps {
        /**
         * The total number of pages.
         */
        pageNum: number;

        /**
         * The range of pages displayed.
         */
        pageRangeDisplayed: number;

        /**
         * The number of pages to display for margins.
         */
        marginPagesDisplayed: number;

        /**
         * Label for the `previous` button.
         */
        previousLabel?: string | JSX.Element;

        /**
         * Label for the `next` button.
         */
        nextLabel?: string | JSX.Element;

        /**
         * Label for ellipsis.
         */
        breakLabel?: string | JSX.Element;

        /**
         * The classname on tag `li` of the ellipsis element.
         */
        breakClassName?: string | JSX.Element;

        /**
         * The method to call when a page is clicked.
         */
        clickCallback?: Function;

        /**
         * The initial page selected.
         */
        initialSelected?: number;

        /**
         * To override selected page with parent prop.
         */
        forceSelected?: number;

        /**
         * The classname of the pagination container.
         */
        containerClassName?: string;

        /**
         * The classname on tag `li` of each page element.
         */
        pageClassName?: string;

        /**
         * The classname on tag `a` of each page element.
         */
        pageLinkClassName?: string;

        /**
         * The classname for the active page.
         */
        activeClassName?: string;

        /**
         * The classname on tag `li` of the `previous` button.
         */
        previousClassName?: string;

        /**
         * The classname on tag `li` of the `next` button.
         */
        nextClassName?: string;

        /**
         * The classname on tag `a` of the `previous` button.
         */
        previousLinkClassName?: string;

        /**
         * The classname on tag `a` of the `next` button.
         */
        nextLinkClassName?: string;

        /**
         * The classname for disabled `previous` and `next` buttons.
         */
        disabledClassName?: string;
    }

    const ReactPaginate: React.ComponentClass<ReactPaginateProps>;
    export = ReactPaginate;
}
